@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.35 2004/09/13 00:05:10 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.38 2004/11/05 11:51:30 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -53,7 +53,7 @@ static void set_nonblock(int fd);
 static void unset_nonblock(int fd);
 #endif
 
-static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd);
+static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, int *is_timeout);
 static StartupPacket *read_startup_packet(POOL_CONNECTION *cp);
 static int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp);
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend);
@@ -61,23 +61,32 @@ static void cancel_request(CancelPacket *sp, int secondary_backend);
 static RETSIGTYPE die(int sig);
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 
+/*
+ * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
+ */
+static int exit_request;
+
+static int idle;		/* non 0 means this child is in idle state */
+
 /*
 * child main loop
 */
 void do_child(int unix_fd, int inet_fd)
 {
-	StartupPacket *sp;
 	POOL_CONNECTION *frontend;
 	POOL_CONNECTION_POOL *backend;
 	struct timeval now;
 	struct timezone tz;
+	int child_idle_sec;
+	int connected;
 
 	pool_debug("I am %d", getpid());
 
 	/* set up signal handlers */
 	signal(SIGALRM, SIG_DFL);
 	signal(SIGTERM, die);
-	signal(SIGINT, SIG_DFL);
+	signal(SIGINT, die);
+	signal(SIGQUIT, die);
 	signal(SIGCHLD, SIG_DFL);
 	signal(SIGUSR1, SIG_DFL);
 	signal(SIGPIPE, SIG_IGN);
@@ -101,14 +110,40 @@ void do_child(int unix_fd, int inet_fd)
 		exit(1);
 	}
 
+	child_idle_sec = 0;
+	connected = 0;
+
 	for (;;)
 	{
 		int connection_reuse = 1;
 		int ssl_request = 0;
+		StartupPacket *sp;
+		int is_timeout = 0;
+
+		if (exit_request)
+		{
+			die(0);
+			exit(0);
+		}
+
+		idle = 1;
 
 		/* perform accept() */
-		frontend = do_accept(unix_fd, inet_fd);
-		if (frontend == NULL)
+		frontend = do_accept(unix_fd, inet_fd, &is_timeout);
+
+		if (connected && is_timeout)
+		{
+			/* select() timeout */
+			child_idle_sec++;
+			if (pool_config.child_life_time != 0 && child_idle_sec >= pool_config.child_life_time)
+			{
+				pool_debug("child life %d expired", pool_config.child_life_time);
+				die(0);
+			}
+			continue;
+		}
+
+		else if (frontend == NULL)
 		{
 			/* accept() failed. return to the accept() loop */
 			continue;
@@ -119,25 +154,19 @@ void do_child(int unix_fd, int inet_fd)
 		unset_nonblock(frontend->fd);
 #endif
 
+		/* set busy flag and clear child idle timer */
+		idle = 0;
+		child_idle_sec = 0;
+
 		/* disable timeout */
 		pool_disable_timeout();
 
 		/* read the startup packet */
-		sp = 0;
 	retry_startup:
-		if (sp)
-		{
-			free(sp->startup_packet);
-			free(sp->database);
-			free(sp->user);
-			free(sp);
-		}
-
 		sp = read_startup_packet(frontend);
 		if (sp == NULL)
 		{
-			/* failed to read the startup packet. return to the
-			   accept() loop */
+			/* failed to read the startup packet. return to the accept() loop */
 			pool_close(frontend);
 			continue;
 		}
@@ -149,6 +178,7 @@ void do_child(int unix_fd, int inet_fd)
 			if (pool_config.replication_enabled)
 				cancel_request((CancelPacket *)sp->startup_packet, 1);
 			pool_close(frontend);
+			pool_free_startup_packet(sp);
 			continue;
 		}
 
@@ -160,6 +190,7 @@ void do_child(int unix_fd, int inet_fd)
 			if (ssl_request)
 			{
 				pool_close(frontend);
+				pool_free_startup_packet(sp);
 				continue;
 			}
 
@@ -169,6 +200,7 @@ void do_child(int unix_fd, int inet_fd)
 			 */
 			pool_write_and_flush(frontend, "N", 1);
 			ssl_request = 1;
+			pool_free_startup_packet(sp);
 			goto retry_startup;
 		}
 
@@ -180,6 +212,8 @@ void do_child(int unix_fd, int inet_fd)
 		 * if there's no connection associated with user and database,
 		 * we need to connect to the backend and send the startup packet.
 		 */
+
+		/* look for existing connection */
 		if ((backend = pool_get_cp(sp->user, sp->database, sp->major)) == NULL)
 		{
 			/* create a new connection to backend */
@@ -190,6 +224,9 @@ void do_child(int unix_fd, int inet_fd)
 		}
 		else
 		{
+			/* we don't need Startup Packet info anymore */
+			pool_free_startup_packet(sp);
+
 			/* reuse existing connection to backend */
 
 			if (pool_do_reauth(frontend, backend))
@@ -233,6 +270,9 @@ void do_child(int unix_fd, int inet_fd)
 		if (pool_config.replication_strict == 0)
 			pool_enable_timeout();
 
+		/* mark this process has accepted connection */
+		connected++;
+
 		/* query process loop */
 		for (;;)
 		{
@@ -240,6 +280,8 @@ void do_child(int unix_fd, int inet_fd)
 
 			status = pool_process_query(frontend, backend, 0);
 
+			sp = MASTER_CONNECTION(backend)->sp;
+
 			switch (status)
 			{
 				/* client exits */
@@ -290,7 +332,7 @@ void do_child(int unix_fd, int inet_fd)
 
 				/* not implemented yet */
 				case POOL_IDLE:
-					do_accept(unix_fd, inet_fd);
+					do_accept(unix_fd, inet_fd, NULL);
 					pool_debug("accept while idle");
 					break;
 
@@ -360,7 +402,7 @@ static void unset_nonblock(int fd)
 /*
 * perform accept() and returns new fd
 */
-static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd)
+static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, int *is_timeout)
 {
     fd_set	readmask;
     int fds;
@@ -376,13 +418,14 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd)
 	static long atime;
 	static int cnt;
 #endif
+	struct timeval timeout = {1, 0};
 
 	FD_ZERO(&readmask);
 	FD_SET(unix_fd, &readmask);
 	if (inet_fd)
 		FD_SET(inet_fd, &readmask);
 
-	fds = select(Max(unix_fd, inet_fd)+1, &readmask, NULL, NULL, NULL);
+	fds = select(Max(unix_fd, inet_fd)+1, &readmask, NULL, NULL, &timeout);
 	if (fds == -1)
 	{
 		if (errno == EAGAIN || errno == EINTR)
@@ -392,8 +435,12 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd)
 		return NULL;
 	}
 
+	/* timeout */
 	if (fds == 0)
+	{
+		*is_timeout = 1;
 		return NULL;
+	}
 
 	if (FD_ISSET(unix_fd, &readmask))
 	{
@@ -479,7 +526,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	int len;
 	char *p;
 
-	sp = (StartupPacket *)malloc(sizeof(StartupPacket));
+	sp = (StartupPacket *)calloc(sizeof(*sp), 1);
 	if (!sp)
 	{
 		pool_error("read_startup_packet: out of memory");
@@ -503,12 +550,14 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	if (!sp->startup_packet)
 	{
 		pool_error("read_startup_packet: out of memory");
+		pool_free_startup_packet(sp);
 		return NULL;
 	}
 
 	/* read startup packet */
 	if (pool_read(cp, sp->startup_packet, len))
 	{
+		pool_free_startup_packet(sp);
 		return NULL;
 	}
 
@@ -527,6 +576,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			if (!sp->database)
 			{
 				pool_error("read_startup_packet: out of memory");
+				pool_free_startup_packet(sp);
 				return NULL;
 			}
 			strncpy(sp->database, sp2->database, SM_DATABASE);
@@ -535,6 +585,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			if (!sp->user)
 			{
 				pool_error("read_startup_packet: out of memory");
+				pool_free_startup_packet(sp);
 				return NULL;
 			}
 			strncpy(sp->user, sp2->user, SM_USER);
@@ -553,6 +604,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 					if (!sp->user)
 					{
 						pool_error("read_startup_packet: out of memory");
+						pool_free_startup_packet(sp);
 						return NULL;
 					}
 				}
@@ -563,6 +615,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 					if (!sp->database)
 					{
 						pool_error("read_startup_packet: out of memory");
+						pool_free_startup_packet(sp);
 						return NULL;
 					}
 				}
@@ -576,18 +629,21 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			if (!sp->database)
 			{
 				pool_error("read_startup_packet: out of memory");
+				pool_free_startup_packet(sp);
 				return NULL;
 			}
 			sp->user = calloc(1, 1);
 			if (!sp->user)
 			{
 				pool_error("read_startup_packet: out of memory");
+				pool_free_startup_packet(sp);
 				return NULL;
 			}
 			break;
 
 		default:
 			pool_error("read_startup_packet: invalid major no: %d", sp->major);
+			pool_free_startup_packet(sp);
 			return NULL;
 	}
 
@@ -664,11 +720,13 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 		pool_send_error_message(frontend, sp->major, "XX000", "connection cache is full", "",
 								"increace max_pool", __FILE__, __LINE__);
 		pool_close(frontend);
+		pool_free_startup_packet(sp);
 		return NULL;
 	}
 
 	/* mark this is a backend connection */
 	backend->slots[0]->con->isbackend = 1;
+
 	/*
 	 * save startup packet info
 	 */
@@ -688,6 +746,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	if (send_startup_packet(backend->slots[0]) < 0)
 	{
 		pool_error("do_child: fails to send startup packet to the backend");
+		pool_discard_cp(sp->user, sp->database, sp->major);
 		pool_close(frontend);
 		return NULL;
 	}
@@ -698,6 +757,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 		if (send_startup_packet(backend->slots[1]) < 0)
 		{
 			pool_error("do_child: fails to send startup packet to the secondary backend");
+			pool_discard_cp(sp->user, sp->database, sp->major);
 			pool_close(frontend);
 			return NULL;
 		}
@@ -717,6 +777,33 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 
 static RETSIGTYPE die(int sig)
 {
+	int i;
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+
+	exit_request = 1;
+
+	switch (sig)
+	{
+		case SIGTERM:	/* smart shutdown */
+			if (idle == 0)
+				return;
+
+		case SIGINT:	/* fast shutdown */
+		case SIGQUIT:	/* immediate shutdown */
+			exit(0);
+			break;
+		default:
+			break;
+	}
+
+	for (i=0;i<pool_config.max_pool;i++, p++)
+	{
+		if (!MASTER_CONNECTION(p))
+			continue;
+		if (MASTER_CONNECTION(p)->sp->user == NULL)
+			continue;
+		pool_send_frontend_exits(p);
+	}
 	exit(0);
 }
 
@@ -744,3 +831,17 @@ static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 	}
 	return 0;
 }
+
+void pool_free_startup_packet(StartupPacket *sp)
+{
+	if (sp)
+	{
+		if (sp->startup_packet)
+			free(sp->startup_packet);
+		if (sp->database)
+			free(sp->database);
+		if (sp->user)
+			free(sp->user);
+		free(sp);
+	}
+}
