@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.20 2004/10/16 00:40:11 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.21 2004/11/05 11:51:30 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -81,6 +81,8 @@ pid_t mypid;
 
 long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 
+static int stop_sig = SIGTERM;
+
 /*
 * pgpool main program
 */
@@ -93,7 +95,7 @@ int main(int argc, char **argv)
 
 	snprintf(conf_file, sizeof(conf_file), "%s/%s", DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
 
-	while ((opt = getopt(argc, argv, "f:hnd")) != -1)
+	while ((opt = getopt(argc, argv, "f:hm:nd")) != -1)
 	{
 		switch (opt)
 		{
@@ -109,6 +111,25 @@ int main(int argc, char **argv)
 				usage();
 				exit(0);
 				break;
+			case 'm':
+				if (!optarg)
+				{
+					usage();
+					exit(1);
+				}
+				if (*optarg == 's' || !strcmp("smart", optarg))
+					stop_sig = SIGTERM;		/* smart shutdown */
+				else if (*optarg == 'f' || !strcmp("fast", optarg))
+					stop_sig = SIGINT;		/* fast shutdown */
+				else if (*optarg == 'i' || !strcmp("immediate", optarg))
+					stop_sig = SIGQUIT;		/* immediate shutdown */
+				else
+				{
+					usage();
+					exit(1);
+				}
+				break;
+				
 			case 'n':
 				not_detach = 1;
 				break;
@@ -140,7 +161,6 @@ int main(int argc, char **argv)
 	if (optind == (argc - 1) && !strcmp(argv[optind], "stop"))
 	{
 		stop_me();
-		fprintf(stderr, "pgpool stopped.\n");
 		exit(0);
 	}
 	else if (optind == argc)
@@ -205,6 +225,7 @@ int main(int argc, char **argv)
 	/* set up signal handlers */
 	signal(SIGTERM, exit_handler);
 	signal(SIGINT, exit_handler);
+	signal(SIGQUIT, exit_handler);
 	signal(SIGCHLD, reap_handler);
 	signal(SIGUSR1, failover_handler);
 
@@ -219,7 +240,7 @@ static void usage(void)
 {
 	fprintf(stderr, "pgpool version %s(%s),\n",	VERSION, PGPOOLVERSION);
 	fprintf(stderr, "  a generic connection pool/replication/load balance server for PostgreSQL\n\n");
-	fprintf(stderr, "usage: pgpool [-f config_file][-n][-d][-h][stop]\n");
+	fprintf(stderr, "usage: pgpool [-f config_file][-n][-d][-h][[-m {s[mart]|f[ast]|i[mmediate]}] stop]\n");
 	fprintf(stderr, "  config_file default path: %s/%s\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
 	fprintf(stderr, "  -n: don't run in daemon mode. does not detatch control tty\n");
 	fprintf(stderr, "  -d: debug mode. lots of debug information will be printed\n");
@@ -288,11 +309,20 @@ static void stop_me(void)
 	pid = atoi(pidbuf);
 	fclose(fd);
 
-	if (kill(pid, SIGTERM) == -1)
+	if (kill(pid, stop_sig) == -1)
 	{
-		pool_error("could stop pid: %d. reason: %s", pid, strerror(errno));
+		pool_error("could not stop pid: %d. reason: %s", pid, strerror(errno));
 		exit(1);
 	}
+
+	fprintf(stderr, "stop request sent to pgpool. waiting for termination...");
+
+	while (kill(pid, 0) == 0)
+	{
+		fprintf(stderr, ".");
+		sleep(1);
+	}
+	fprintf(stderr, "done.\n");
 }
 
 /*
@@ -522,7 +552,7 @@ static RETSIGTYPE exit_handler(int sig)
 		pid_t pid = pids[i];
 		if (pid)
 		{
-			kill(pid, SIGTERM);
+			kill(pid, sig);
 		}
 	}
 	while (wait(NULL) > 0)
