@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.24 2004/08/18 07:05:17 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.25 2004/11/04 11:06:40 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -106,9 +106,7 @@ void pool_discard_cp(char *user, char *database, int protoMajor)
 		return;
 	}
 
-	free(MASTER_CONNECTION(p)->sp->user);
-	free(MASTER_CONNECTION(p)->sp->database);
-	free(MASTER_CONNECTION(p)->sp->startup_packet);
+	pool_free_startup_packet(MASTER_CONNECTION(p)->sp);
 	pool_close(MASTER_CONNECTION(p)->con);
 
 	if (pool_config.replication_enabled)
@@ -174,9 +172,7 @@ POOL_CONNECTION_POOL *pool_create_cp(void)
 			   MASTER_CONNECTION(p)->sp->user,
 			   MASTER_CONNECTION(p)->sp->database);
 
-	free(MASTER_CONNECTION(p)->sp->user);
-	free(MASTER_CONNECTION(p)->sp->database);
-	free(MASTER_CONNECTION(p)->sp->startup_packet);
+	pool_free_startup_packet(MASTER_CONNECTION(p)->sp);
 	pool_close(MASTER_CONNECTION(p)->con);
 
 	if (pool_config.replication_enabled)
@@ -259,9 +255,7 @@ RETSIGTYPE pool_backend_timer_handler(int sig)
 
 				pool_send_frontend_exits(p);
 
-				free(MASTER_CONNECTION(p)->sp->user);
-				free(MASTER_CONNECTION(p)->sp->database);
-				free(MASTER_CONNECTION(p)->sp->startup_packet);
+				pool_free_startup_packet(MASTER_CONNECTION(p)->sp);
 				pool_close(MASTER_CONNECTION(p)->con);
 
 				if (pool_config.replication_enabled)
