@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.32 2004/09/30 12:34:11 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.34 2004/10/28 03:54:03 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -283,25 +283,32 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 			}
 			if (kind1 == 0)
 			{
-				/* at this point the query should have completed and it's safe to set timeout here */
-				pool_debug("pool_process_query: waiting for secondary for data ready");
-
-				/* temporary enable timeout */
-				pool_enable_timeout();
-
-				if (pool_check_fd(SECONDARY(backend), 0))
+				if (SECONDARY(backend)->len <= 0)
 				{
-					pool_error("pool_process_query: secondary data is not ready at synchronous point. abort this session");
+					/* at this point the query should have completed and it's safe to set timeout here */
+					pool_debug("pool_process_query: waiting for secondary for data ready");
 
+					/* temporary enable timeout */
+					pool_enable_timeout();
+
+					if (pool_check_fd(SECONDARY(backend), 0))
+					{
+						pool_error("pool_process_query: secondary data is not ready at synchronous point. abort this session");
+
+					}
+					else
+					{
+						pool_read(SECONDARY(backend), &kind1, 1);
+					}
+
+					pool_disable_timeout();
 				}
 				else
 				{
-					pool_read(SECONDARY(backend), &kind1, 1);
+						pool_read(SECONDARY(backend), &kind1, 1);
 				}
-
-				pool_disable_timeout();
-
 			}
+
 			if (kind == '\0' || kind != kind1)
 			{
 				pool_error("pool_process_query: kind does not match between backends master(%c) secondary(%c)",
@@ -1417,8 +1424,8 @@ int pool_check_fd(POOL_CONNECTION *cp, int notimeout)
 			tp = NULL;
 		else
 		{
-			timeout.tv_sec = 0;
-			timeout.tv_usec = pool_config.replication_timeout*1000;
+			timeout.tv_sec = pool_config.replication_timeout / 1000;
+			timeout.tv_usec = pool_config.replication_timeout - (timeout.tv_sec * 1000);
 			tp = &timeout;
 		}
 
@@ -1441,7 +1448,9 @@ int pool_check_fd(POOL_CONNECTION *cp, int notimeout)
 
 		if (fds == 0)
 		{
-			pool_error("pool_check_fd: data is not ready tp->tv_sec %d tp->tp_usec %d", tp->tv_sec, tp->tv_usec);
+			pool_error("pool_check_fd: data is not ready tp->tv_sec %d tp->tp_usec %d", 
+					   pool_config.replication_timeout / 1000,
+					   pool_config.replication_timeout - (timeout.tv_sec * 1000));
 			break;
 		}
 		return 0;
